@@ -2,8 +2,15 @@ from .molgrid import MolGrid
 from .dispatch import (display,
                        save)
 from .select import (get_selection,
-                     selection,
                      list_grids)
 from .utils import (sdf_to_dataframe,
                     make_popup_callback)
 from ._version import __version__
+from .widget import _jupyter_labextension_paths, _jupyter_nbextension_paths
+
+try:
+    from google.colab import output
+except ImportError:
+    pass
+else:
+    output.enable_custom_widget_manager()
